from shutil import move
import numpy as np
import pandas as pd
import sqlite3
from pathlib import Path
from time import sleep

from os.path import basename
from inference.GracefulKiller import GracefulKiller
from contextlib import closing
from helper_functions import safe_cursor

class ReadManager(object):
    """
    Load reads into table, remove if prediction complete
    """

    def __init__(self, index_table_fn, reads_dir, pos_reads_dir, kmers):
        self.conn = sqlite3.connect(index_table_fn)
        # self.cursor = self.conn.cursor()
        self.kmers = kmers
        self.pos_check_total = len(self.kmers)
        self.pos_check_limit = self.pos_check_total // 2
        self.neg_check_limit = len(self.kmers) - self.pos_check_limit
        self.table_columns = ['read_id', 'timestamp'] + self.kmers
        self.reads_dir = Path(reads_dir)
        self.pos_reads_dir = pos_reads_dir
        self.current_reads_list = []
        self.cur_timestamp = 1
        self.run()

    def run(self):
        killer = GracefulKiller()
        while not killer.kill_now:
            self.register_new_predictions()
            self.add_new_reads()

    def register_new_predictions(self):
        # retrieve new predictions
        new_predictions = safe_cursor(self.conn, f"SELECT * FROM read_table where timestamp > {self.cur_timestamp} ")
        if not len(new_predictions): return

        # register
        new_preds_df = pd.DataFrame(new_predictions, columns=self.table_columns).set_index('read_id')
        for fn, tup in new_preds_df.iterrows():
            if np.sum(tup.loc[self.kmers] > 0) >= self.pos_check_limit:  # if more than limit positive: confirmed positive
            # if tup.loc[self.kmers].sum() == self.pos_check_total:  # If all positive: move to confirmed positives and remove from table
                move(fn, self.pos_reads_dir + basename(fn))
                safe_cursor(self.conn, f"DELETE from read_table WHERE read_id = '{fn}'", read_only=False)
            elif np.sum(tup.loc[self.kmers] < 0) > self.neg_check_limit:
            # elif np.any(tup.loc[self.kmers] < 0):  # if any negative: delete read and remove from table
                Path(fn).unlink(missing_ok=True)
                safe_cursor(self.conn, f"DELETE from read_table WHERE read_id = '{fn}'", read_only=False)

        # update timestamp
        self.cur_timestamp = new_preds_df.timestamp.max()

    def add_new_reads(self):
        new_read_list = []
        for rn in self.reads_dir.iterdir():
            if rn not in self.current_reads_list: new_read_list.append(rn)
            if len(new_read_list) >= 10: break

        # todo: test, limit nb of new reads to add at once
        for nrl in new_read_list:
            safe_cursor(self.conn, f"INSERT INTO read_table VALUES ('{str(nrl)}', 0, {', '.join([str(0)] * len(self.kmers))} )",
                        read_only=False)
        self.current_reads_list.extend(new_read_list)
